==> antos0.48.tar.gz: antos0.48/kern/shell.c <==
/*
 * shell.c
 *
 * Copyright 200@, Minoru Murashima. All rights reserved.
 * Distributed under the terms of the BSD License.
 *
 *
 */


#include <sys/config.h>
#include <sys/types.h>
#include <sys/param.h>
#include <kern/mm.h>
#include <kern/fs.h>
#include <kern/loader.h>

#include <kern/test.h>
#include <kern/debug.h>


//#define DEBUG_SHELL 1
#ifdef DEBUG_SHELL
	#define STATIC
	#define INLINE
#else
	#define STATIC	static
	#define INLINE	inline
#endif


//================================== ʒ` ===========================================

//================================== O[oC|[g ===============================

//================================== PRIVATE ============================================

//================================== PROTECTED ==========================================

//================================== PUBLIC =============================================

#define SHELL_MAGIC "#!"

int loadShell(
	EXEC_FD *fd,
	const char *path,		// st@CpX
	char **argv,
	char **envp,
	uint *o_entryAddr,		// X^bNJnjAAhX
	uint *o_stackAddr)		// sJnjAAhX
{
	enum {
		SHELL_MAGIC_SIZE = 2,	// VF}WbNLoCgTCY
	};
	char *header;
	char **argvSehll;
	const char *shellPath;
	int size;
	int last;
	int error;

	/* wb_ǂݍ */
	header = kmalloc(SHELL_MAGIC_SIZE + PATH_MAX + 1);
	if (header == NULL) {
		return -ENOMEM;
	}

	fd->offset = 0;
	size = exec_read(fd, header, SHELL_MAGIC_SIZE + PATH_MAX);
	if (size < SHELL_MAGIC_SIZE) {
		error = (0 <= size) ? -ENOEXEC : size;
		goto ERR;
	}

	// }WbNio[mF
	if (memcmp(SHELL_MAGIC, header, SHELL_MAGIC_SIZE) != 0) {
		error = -ENOEXEC;
		goto ERR;
	}

	// VFvOpX̐ݒ
	for (last = SHELL_MAGIC_SIZE; last < size; ++last) {
		if (header[last] == '\n') {
			break;
		}
	}
	header[last] = '\0';
	shellPath = &header[SHELL_MAGIC_SIZE];

	// |C^XgɃVFt@CpXǉ
	last = 0;
	while (argv[last++] != NULL);
	argvSehll = kmalloc(sizeof(char*) * last + 1);
	if (argvSehll == NULL) {
		error = -ENOMEM;
		goto ERR;
	}
	memcpy(argvSehll + 1, argv, sizeof(char*) * last);
	argvSehll[0] = (char*) path;
	
	error = loadFile(shellPath, argvSehll, envp, o_entryAddr, o_stackAddr);

	kfree(argvSehll);
ERR:
	kfree(header);

	return error;
}

==> antos0.49.tar.gz: antos0.49/kern/shell.c <==
/*
 * shell.c
 *
 * Copyright 200@, Minoru Murashima. All rights reserved.
 * Distributed under the terms of the BSD License.
 *
 *
 */


#include <sys/config.h>
#include <sys/types.h>
#include <sys/param.h>
#include <kern/mm.h>
#include <kern/fs.h>
#include <kern/loader.h>

#include <kern/debug.h>


//#define DEBUG_SHELL 1
#ifdef DEBUG_SHELL
	#define STATIC
	#define INLINE
#else
	#define STATIC	static
	#define INLINE	inline
#endif


//==================================  ===========================================

//================================== Х륤ݡ ===============================

//================================== PRIVATE ============================================

//================================== PROTECTED ==========================================

//================================== PUBLIC =============================================

#define SHELL_MAGIC "#!"

int loadShell(
	EXEC_FD *fd,
	const char *path,		// ¹ԥեѥ
	char **argv,
	char **envp,
	uint *o_entryAddr,		// åϥ˥ɥ쥹
	uint *o_stackAddr)		// ¹Գϥ˥ɥ쥹
{
	enum {
		SHELL_MAGIC_SIZE = 2,	// ޥåХȥ
	};
	char *header;
	char **argvSehll;
	const char *shellPath;
	int size;
	int last;
	int error;

	/* إåɤ߹ */
	header = kmalloc(SHELL_MAGIC_SIZE + PATH_MAX + 1);
	if (header == NULL) {
		return -ENOMEM;
	}

	fd->offset = 0;
	size = exec_read(fd, header, SHELL_MAGIC_SIZE + PATH_MAX);
	if (size < SHELL_MAGIC_SIZE) {
		error = (0 <= size) ? -ENOEXEC : size;
		goto ERR;
	}

	// ޥåʥСǧ
	if (memcmp(SHELL_MAGIC, header, SHELL_MAGIC_SIZE) != 0) {
		error = -ENOEXEC;
		goto ERR;
	}

	// ץѥ
	for (last = SHELL_MAGIC_SIZE; last < size; ++last) {
		if (header[last] == '\n') {
			break;
		}
	}
	header[last] = '\0';
	shellPath = &header[SHELL_MAGIC_SIZE];

	// ݥ󥿥ꥹȤ˥եѥɲä
	last = 0;
	while (argv[last++] != NULL);
	argvSehll = kmalloc(sizeof(char*) * last + 1);
	if (argvSehll == NULL) {
		error = -ENOMEM;
		goto ERR;
	}
	memcpy(argvSehll + 1, argv, sizeof(char*) * last);
	argvSehll[0] = (char*) path;
	
	error = loadFile(shellPath, argvSehll, envp, o_entryAddr, o_stackAddr);

	kfree(argvSehll);
ERR:
	kfree(header);

	return error;
}
